<?php

namespace Drp\UserManagement\Test\Feature;

use Carbon\Carbon;
use Drp\UserManagement\Contracts\TokenGenerator;
use Drp\UserManagement\Events\UserCreated;
use Drp\UserManagement\Events\UserCreatedWithLoginCode;
use Drp\UserManagement\Test\TestCase;
use Drp\UserManagement\User;
use Drp\UserManagement\UserCreator;
use Tests\Support\Fakes\FakeTokenGenerator;

/**
 * Class UserCreatorTest
 * @package Drp\UserManagement\Test\Feature
 */
class UserCreatorTest extends TestCase
{

    public function setup()
    {
        parent::setUp();
        app()->bind(TokenGenerator::class, FakeTokenGenerator::class);
    }

    /**
     * @test
     */
    public function it_creates_a_new_user_with_a_login_token()
    {
        $this->expectsEvents(UserCreatedWithLoginCode::class);

        Carbon::setTestNow(Carbon::create(2017, 01, 01));

        app(UserCreator::class)
            ->createWithLoginCode([
                'name' => 'Mr test',
                'email' => 'mrtest@email.com',
            ]);

        $this->seeInDatabase('users', [
            'id' => 1,
            'name' => 'Mr test',
            'email' => 'mrtest@email.com',
            'first_login_token' => 'TOKEN',
            'login_link_expiry' => Carbon::create(2017, 1, 8)->toDateTimeString(),
        ]);
    }

    /**
     * @test
     */
    public function it_can_regenerate_first_time_login_tokens()
    {
        $this->expectsEvents(UserCreatedWithLoginCode::class);

        $user = factory(User::class)->create();

        Carbon::setTestNow(Carbon::create(2017, 01, 10));

        app(UserCreator::class)
            ->reIssueLoginCode($user);

        $this->seeInDatabase('users', [
            'id' => 1,
            'first_login_token' => 'TOKEN',
            'login_link_expiry' => Carbon::create(2017, 1, 17)->toDateTimeString(),
        ]);
    }


    /**
     * @test
     */
    public function it_creates_a_new_user_without_a_login_token()
    {
        $this->expectsEvents(UserCreated::class);

        Carbon::setTestNow(Carbon::create(2017, 01, 01));

        app(UserCreator::class)
            ->createStandardUser([
                'name' => 'Mr test',
                'email' => 'mrtest@email.com',
            ]);

        $this->seeInDatabase('users', [
            'id' => 1,
            'name' => 'Mr test',
            'email' => 'mrtest@email.com',
            'first_login_token' => null,
            'login_link_expiry' => null,
        ]);
    }

    /**
     * @test
     */
    public function users_are_approved_if_they_do_not_require_approval()
    {
        $this->withoutEvents();

        config()->set('user_management.requires_approval', false);

        app(UserCreator::class)
            ->createStandardUser([
                'name' => 'Mr test',
                'email' => 'mrtest@email.com',
            ]);

        $this->seeInDatabase('users', [
            'id' => 1,
            'approved' => 1
        ]);

        app(UserCreator::class)
            ->createWithLoginCode([
                'name' => 'Mr test2',
                'email' => 'mrtest2@email.com',
            ]);

        $this->seeInDatabase('users', [
            'id' => 2,
            'approved' => 1
        ]);

        config()->set('user_management.requires_approval', true);

        app(UserCreator::class)
            ->createStandardUser([
                'name' => 'Mr test3',
                'email' => 'mrtest3@email.com',
            ]);

        $this->seeInDatabase('users', [
            'id' => 3,
            'approved' => 0
        ]);
    }
}
